@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -43,9 +44,9 @@ struct cipher_table_s {
     size_t blocksize;
     size_t keylen;
     size_t contextsize; /* allocate this amount of context */
-    int  (*setkey)( void *c, byte *key, unsigned keylen );
-    void (*encrypt)( void *c, byte *outbuf, byte *inbuf );
-    void (*decrypt)( void *c, byte *outbuf, byte *inbuf );
+    int  (*setkey)( void *c, const byte *key, unsigned keylen );
+    void (*encrypt)( void *c, byte *outbuf, const byte *inbuf );
+    void (*decrypt)( void *c, byte *outbuf, const byte *inbuf );
 };
 
 static struct cipher_table_s cipher_table[TABLE_SIZE];
@@ -59,9 +60,9 @@ struct cipher_handle_s {
     byte iv[MAX_BLOCKSIZE];	/* (this should be ulong aligned) */
     byte lastiv[MAX_BLOCKSIZE];
     int  unused;  /* in IV */
-    int  (*setkey)( void *c, byte *key, unsigned keylen );
-    void (*encrypt)( void *c, byte *outbuf, byte *inbuf );
-    void (*decrypt)( void *c, byte *outbuf, byte *inbuf );
+    int  (*setkey)( void *c, const byte *key, unsigned keylen );
+    void (*encrypt)( void *c, byte *outbuf, const byte *inbuf );
+    void (*decrypt)( void *c, byte *outbuf, const byte *inbuf );
     PROPERLY_ALIGNED_TYPE context;
 };
 
@@ -85,43 +86,45 @@ dummy_decrypt_block( void *c, byte *outbuf, byte *inbuf ) { BUG(); }
 static void
 setup_cipher_table(void)
 {
+    int i=0;
 
-    int i;
+#ifdef USE_AES
+    cipher_table[i].algo = CIPHER_ALGO_AES;
+    cipher_table[i].name = rijndael_get_info( cipher_table[i].algo,
+					 &cipher_table[i].keylen,
+					 &cipher_table[i].blocksize,
+					 &cipher_table[i].contextsize,
+					 &cipher_table[i].setkey,
+					 &cipher_table[i].encrypt,
+					 &cipher_table[i].decrypt     );
+    if( !cipher_table[i].name )
+	BUG();
+    i++;
+    cipher_table[i].algo = CIPHER_ALGO_AES192;
+    cipher_table[i].name = rijndael_get_info( cipher_table[i].algo,
+					 &cipher_table[i].keylen,
+					 &cipher_table[i].blocksize,
+					 &cipher_table[i].contextsize,
+					 &cipher_table[i].setkey,
+					 &cipher_table[i].encrypt,
+					 &cipher_table[i].decrypt     );
+    if( !cipher_table[i].name )
+	BUG();
+    i++;
+    cipher_table[i].algo = CIPHER_ALGO_AES256;
+    cipher_table[i].name = rijndael_get_info( cipher_table[i].algo,
+					 &cipher_table[i].keylen,
+					 &cipher_table[i].blocksize,
+					 &cipher_table[i].contextsize,
+					 &cipher_table[i].setkey,
+					 &cipher_table[i].encrypt,
+					 &cipher_table[i].decrypt     );
+    if( !cipher_table[i].name )
+	BUG();
+    i++;
+#endif
 
-    i = 0;
-    cipher_table[i].algo = CIPHER_ALGO_RIJNDAEL;
-    cipher_table[i].name = rijndael_get_info( cipher_table[i].algo,
-					 &cipher_table[i].keylen,
-					 &cipher_table[i].blocksize,
-					 &cipher_table[i].contextsize,
-					 &cipher_table[i].setkey,
-					 &cipher_table[i].encrypt,
-					 &cipher_table[i].decrypt     );
-    if( !cipher_table[i].name )
-	BUG();
-    i++;
-    cipher_table[i].algo = CIPHER_ALGO_RIJNDAEL192;
-    cipher_table[i].name = rijndael_get_info( cipher_table[i].algo,
-					 &cipher_table[i].keylen,
-					 &cipher_table[i].blocksize,
-					 &cipher_table[i].contextsize,
-					 &cipher_table[i].setkey,
-					 &cipher_table[i].encrypt,
-					 &cipher_table[i].decrypt     );
-    if( !cipher_table[i].name )
-	BUG();
-    i++;
-    cipher_table[i].algo = CIPHER_ALGO_RIJNDAEL256;
-    cipher_table[i].name = rijndael_get_info( cipher_table[i].algo,
-					 &cipher_table[i].keylen,
-					 &cipher_table[i].blocksize,
-					 &cipher_table[i].contextsize,
-					 &cipher_table[i].setkey,
-					 &cipher_table[i].encrypt,
-					 &cipher_table[i].decrypt     );
-    if( !cipher_table[i].name )
-	BUG();
-    i++;
+#ifdef USE_TWOFISH
     cipher_table[i].algo = CIPHER_ALGO_TWOFISH;
     cipher_table[i].name = twofish_get_info( cipher_table[i].algo,
 					 &cipher_table[i].keylen,
@@ -133,6 +136,9 @@ setup_cipher_table(void)
     if( !cipher_table[i].name )
 	BUG();
     i++;
+#endif
+
+#ifdef USE_BLOWFISH
     cipher_table[i].algo = CIPHER_ALGO_BLOWFISH;
     cipher_table[i].name = blowfish_get_info( cipher_table[i].algo,
 					 &cipher_table[i].keylen,
@@ -144,6 +150,9 @@ setup_cipher_table(void)
     if( !cipher_table[i].name )
 	BUG();
     i++;
+#endif
+
+#ifdef USE_CAST5
     cipher_table[i].algo = CIPHER_ALGO_CAST5;
     cipher_table[i].name = cast5_get_info( cipher_table[i].algo,
 					 &cipher_table[i].keylen,
@@ -155,6 +164,8 @@ setup_cipher_table(void)
     if( !cipher_table[i].name )
 	BUG();
     i++;
+#endif
+
     cipher_table[i].algo = CIPHER_ALGO_3DES;
     cipher_table[i].name = des_get_info( cipher_table[i].algo,
 					 &cipher_table[i].keylen,
@@ -166,16 +177,20 @@ setup_cipher_table(void)
     if( !cipher_table[i].name )
 	BUG();
     i++;
+
+#ifdef USE_IDEA
     cipher_table[i].algo = CIPHER_ALGO_IDEA;
     cipher_table[i].name = idea_get_info( cipher_table[i].algo,
-					 &cipher_table[i].keylen,
-					 &cipher_table[i].blocksize,
-					 &cipher_table[i].contextsize,
-					 &cipher_table[i].setkey,
-					 &cipher_table[i].encrypt,
-					 &cipher_table[i].decrypt     );
+					  &cipher_table[i].keylen,
+					  &cipher_table[i].blocksize,
+					  &cipher_table[i].contextsize,
+					  &cipher_table[i].setkey,
+					  &cipher_table[i].encrypt,
+					  &cipher_table[i].decrypt     );
     if (cipher_table[i].name)
-      i++;  /* Note that IDEA is usually not available. */
+      i++;  /* Note that the loadable IDEA module may not be
+	       available. */
+#endif
 
 #ifdef ALLOW_DUMMY
     cipher_table[i].algo = CIPHER_ALGO_DUMMY;
@@ -211,12 +226,6 @@ load_cipher_modules(void)
   return 0;
 }
 
-
-
-
-
-
-
 /****************
  * Map a string to the cipher algo
  */
@@ -245,6 +254,20 @@ string_to_cipher_algo( const char *string )
             return cipher_table[i].algo;
         }
     } while( load_cipher_modules() );
+
+    /* Didn't find it, so try the Sx format */
+    if(string[0]=='S' || string[0]=='s')
+      {
+	long val;
+	char *endptr;
+
+	string++;
+
+	val=strtol(string,&endptr,10);
+	if(*string!='\0' && *endptr=='\0' && check_cipher_algo(val)==0)
+	  return val;
+      }
+
   return 0;
 }
 
@@ -366,10 +389,10 @@ cipher_open( int algo, int mode, int secure )
 
     /* ? perform selftest here and mark this with a flag in cipher_table ? */
 
-    hd = secure ? m_alloc_secure_clear( sizeof *hd
+    hd = secure ? xmalloc_secure_clear( sizeof *hd
 					+ cipher_table[i].contextsize
 					- sizeof(PROPERLY_ALIGNED_TYPE) )
-		: m_alloc_clear( sizeof *hd + cipher_table[i].contextsize
+		: xmalloc_clear( sizeof *hd + cipher_table[i].contextsize
 					   - sizeof(PROPERLY_ALIGNED_TYPE)  );
     hd->algo = algo;
     hd->blocksize = cipher_table[i].blocksize;
@@ -398,7 +421,7 @@ cipher_open( int algo, int mode, int secure )
 void
 cipher_close( CIPHER_HANDLE c )
 {
-    m_free(c);
+    xfree(c);
 }
 
 
@@ -409,7 +432,6 @@ cipher_setkey( CIPHER_HANDLE c, byte *key, unsigned keylen )
 }
 
 
-
 void
 cipher_setiv( CIPHER_HANDLE c, const byte *iv, unsigned ivlen )
 {
@@ -425,8 +447,6 @@ cipher_setiv( CIPHER_HANDLE c, const byte *iv, unsigned ivlen )
     c->unused = 0;
 }
 
-
-
 static void
 do_ecb_encrypt( CIPHER_HANDLE c, byte *outbuf, byte *inbuf, unsigned nblocks )
 {
@@ -516,7 +536,25 @@ do_cfb_encrypt( CIPHER_HANDLE c, byte *outbuf, byte *inbuf, unsigned nbytes )
 	    *outbuf++ = (*ivp++ ^= *inbuf++);
     }
 
-    /* now we can process complete blocks */
+    /* Now we can process complete blocks. */
+#if 0 
+    /* Experimental code.  We may only use this for standard CFB
+       because for Phil's mode we need to save the IV of before the
+       last encryption - we don't want to do this in tghe fasf CFB
+       encryption routine.  */
+    if (c->algo == CIPHER_ALGO_AES
+        && nbytes >= blocksize 
+        && c->mode != CIPHER_MODE_PHILS_CFB) {
+        size_t n;
+
+	memcpy( c->lastiv, c->iv, blocksize );
+        n = (nbytes / blocksize) * blocksize;
+        rijndael_cfb_encrypt (&c->context.c, c->iv, outbuf, inbuf, n);
+        inbuf  += n;
+        outbuf += n;
+	nbytes -= n;
+    }
+#endif
     while( nbytes >= blocksize ) {
 	int i;
 	/* encrypt the IV (and save the current one) */
@@ -675,4 +713,3 @@ cipher_sync( CIPHER_HANDLE c )
 	c->unused = 0;
     }
 }
-
