@@ -15,13 +15,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
-
-#ifdef USE_RNDEGD
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -139,7 +137,7 @@ rndegd_connect_socket (int nofail)
       close (fd);
       fd = -1;
     }
-  m_free(name);
+  xfree(name);
   if (fd != -1)
     egd_socket = fd;
   return fd;
@@ -225,5 +223,3 @@ rndegd_gather_random( void (*add)(const void*, size_t, int), int requester,
 
     return 0; /* success */
 }
-
-#endif /*USE_RNDEGD*/
