@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -112,7 +113,6 @@ get_status_string ( int no )
     case STATUS_ENC_TO	 : s = "ENC_TO"; break;
     case STATUS_NODATA	 : s = "NODATA"; break;
     case STATUS_BAD_PASSPHRASE : s = "BAD_PASSPHRASE"; break;
-    case STATUS_BAD_PASSPHRASE_PIN: s = "BAD_PASSPHRASE_PIN"; break;
     case STATUS_NO_PUBKEY	 : s = "NO_PUBKEY"; break;
     case STATUS_NO_SECKEY	 : s = "NO_SECKEY"; break;
     case STATUS_NEED_PASSPHRASE_SYM: s = "NEED_PASSPHRASE_SYM"; break;
@@ -163,6 +163,9 @@ get_status_string ( int no )
     case STATUS_SC_OP_SUCCESS  : s = "SC_OP_SUCCESS"; break;
     case STATUS_SC_OP_FAILURE  : s = "SC_OP_FAILURE"; break;
     case STATUS_BACKUP_KEY_CREATED:s="BACKUP_KEY_CREATED"; break;
+    case STATUS_PKA_TRUST_BAD  : s = "PKA_TRUST_BAD"; break;
+    case STATUS_PKA_TRUST_GOOD : s = "PKA_TRUST_GOOD"; break;
+    case STATUS_BEGIN_SIGNING  : s = "BEGIN_SIGNING"; break;
     default: s = "?"; break;
     }
   return s;
@@ -263,7 +266,8 @@ write_status_text ( int no, const char *text)
         }
     }
     putc ('\n',statusfp);
-    fflush (statusfp);
+    if ( fflush (statusfp) && opt.exit_on_status_write_error )
+      g10_exit (0);
 }
 
 
@@ -327,7 +331,8 @@ write_status_text_and_buffer ( int no, const char *string,
     } while ( len );
 
     putc ('\n',statusfp);
-    fflush (statusfp);
+    if ( fflush (statusfp) && opt.exit_on_status_write_error )
+      g10_exit (0);
 }
 
 void
@@ -499,7 +504,7 @@ do_shm_get( const char *keyword, int hidden, int bool )
     if( bool )
 	return p[0]? "" : NULL;
 
-    string = hidden? m_alloc_secure( n+1 ) : m_alloc( n+1 );
+    string = hidden? xmalloc_secure( n+1 ) : xmalloc( n+1 );
     memcpy(string, p, n );
     string[n] = 0; /* make sure it is a string */
     if( hidden ) /* invalidate the memory */
@@ -557,7 +562,7 @@ do_get_from_fd( const char *keyword, int hidden, int bool )
 	if( i >= len-1 ) {
 	    char *save = string;
 	    len += 100;
-	    string = hidden? m_alloc_secure ( len ) : m_alloc ( len );
+	    string = hidden? xmalloc_secure ( len ) : xmalloc ( len );
 	    if( save )
 		memcpy(string, save, i );
 	    else
@@ -628,7 +633,7 @@ cpr_get( const char *keyword, const char *prompt )
     for(;;) {
 	p = tty_get( prompt );
 	if( *p=='?' && !p[1] && !(keyword && !*keyword)) {
-	    m_free(p);
+	    xfree(p);
 	    display_online_help( keyword );
 	}
 	else
@@ -644,7 +649,7 @@ cpr_get_utf8( const char *keyword, const char *prompt )
     p = cpr_get( keyword, prompt );
     if( p ) {
 	char *utf8 = native_to_utf8( p );
-	m_free( p );
+	xfree( p );
 	p = utf8;
     }
     return p;
@@ -664,7 +669,7 @@ cpr_get_hidden( const char *keyword, const char *prompt )
     for(;;) {
 	p = tty_get_hidden( prompt );
 	if( *p == '?' && !p[1] ) {
-	    m_free(p);
+	    xfree(p);
 	    display_online_help( keyword );
 	}
 	else
@@ -701,13 +706,13 @@ cpr_get_answer_is_yes( const char *keyword, const char *prompt )
 	p = tty_get( prompt );
 	trim_spaces(p); /* it is okay to do this here */
 	if( *p == '?' && !p[1] ) {
-	    m_free(p);
+	    xfree(p);
 	    display_online_help( keyword );
 	}
 	else {
 	    tty_kill_prompt();
 	    yes = answer_is_yes(p);
-	    m_free(p);
+	    xfree(p);
 	    return yes;
 	}
     }
@@ -729,13 +734,13 @@ cpr_get_answer_yes_no_quit( const char *keyword, const char *prompt )
 	p = tty_get( prompt );
 	trim_spaces(p); /* it is okay to do this here */
 	if( *p == '?' && !p[1] ) {
-	    m_free(p);
+	    xfree(p);
 	    display_online_help( keyword );
 	}
 	else {
 	    tty_kill_prompt();
 	    yes = answer_is_yes_no_quit(p);
-	    m_free(p);
+	    xfree(p);
 	    return yes;
 	}
     }
@@ -761,7 +766,7 @@ cpr_get_answer_okay_cancel (const char *keyword,
   if (answer)
     {
       yes = answer_is_okay_cancel (answer, def_answer);
-      m_free (answer);
+      xfree (answer);
       return yes;
     }
 
@@ -771,14 +776,14 @@ cpr_get_answer_okay_cancel (const char *keyword,
       trim_spaces(p); /* it is okay to do this here */
       if (*p == '?' && !p[1])
         {
-          m_free(p);
+          xfree(p);
           display_online_help (keyword);
 	}
       else
         {
           tty_kill_prompt();
           yes = answer_is_okay_cancel (p, def_answer);
-          m_free(p);
+          xfree(p);
           return yes;
 	}
     }
