@@ -30,7 +30,7 @@
 #include "errors.h"
 #include "types.h"
 #include "mpi.h"
-
+#include "compat.h"
 
 typedef struct {
      int  *argc;	    /* pointer to argc (value subject to change) */
@@ -164,7 +164,6 @@ int answer_is_yes( const char *s );
 int answer_is_yes_no_quit( const char *s );
 int answer_is_okay_cancel (const char *s, int def_answer);
 int match_multistr(const char *multistr,const char *match);
-int hextobyte( const char *s );
 
 /*-- strgutil.c --*/
 void free_strlist( STRLIST sl );
@@ -207,9 +206,6 @@ char *stpcpy(char *a,const char *b);
 #ifndef HAVE_STRLWR
 char *strlwr(char *a);
 #endif
-#ifndef HAVE_STRSEP
-char *strsep (char **stringp, const char *delim);
-#endif
 #ifndef HAVE_STRCASECMP
 int strcasecmp( const char *, const char *b);
 #endif
@@ -298,10 +294,6 @@ int get_cert(const char *name,size_t max_size,IOBUF *iobuf,
                      *(p) <= 'F'? (*(p)-'A'+10):(*(p)-'a'+10))
 #define xtoi_2(p)   ((xtoi_1(p) * 16) + xtoi_1((p)+1))
 
-/* Note this isn't identical to a C locale isspace() without \f and
-   \v, but works for the purposes used here. */
-#define ascii_isspace(a) ((a)==' ' || (a)=='\n' || (a)=='\r' || (a)=='\t')
-
 /******* RISC OS stuff ***********/
 #ifdef __riscos__
 int riscos_load_module(const char *name, const char * const path[], int fatal);
