@@ -1,5 +1,6 @@
 /* dotlock.c - dotfile locking
- *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2004,
+ *               2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -25,11 +27,13 @@
 #include <ctype.h>
 #include <errno.h>
 #include <unistd.h>
-#ifndef  HAVE_DOSISH_SYSTEM
+#if !defined (HAVE_DOSISH_SYSTEM)
 #include <sys/utsname.h>
 #endif
 #include <sys/types.h>
+#ifndef _WIN32
 #include <sys/time.h>
+#endif
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -49,8 +53,6 @@ struct dotlock_handle {
 static volatile DOTLOCK all_lockfiles;
 static int never_lock;
 
-static int read_lockfile( const char *name );
-
 void
 disable_dotlock(void)
 {
@@ -78,14 +80,14 @@ create_dotlock( const char *file_to_lock )
 {
     static int initialized;
     DOTLOCK h;
-    int  fd = -1;
+#if !defined (HAVE_DOSISH_SYSTEM)
+    int fd = -1;
     char pidstr[16];
-  #ifndef  HAVE_DOSISH_SYSTEM
     struct utsname utsbuf;
-  #endif
     const char *nodename;
     const char *dirpart;
     int dirpartlen;
+#endif
 
     if( !initialized ) {
 	atexit( remove_lockfiles );
@@ -94,19 +96,19 @@ create_dotlock( const char *file_to_lock )
     if( !file_to_lock )
 	return NULL;
 
-    h = m_alloc_clear( sizeof *h );
+    h = xmalloc_clear( sizeof *h );
     if( never_lock ) {
 	h->disable = 1;
-      #ifdef _REENTRANT
+#ifdef _REENTRANT
 	/* fixme: aquire mutex on all_lockfiles */
-      #endif
+#endif
 	h->next = all_lockfiles;
 	all_lockfiles = h;
 	return h;
     }
 
 
-#ifndef HAVE_DOSISH_SYSTEM
+#if !defined (HAVE_DOSISH_SYSTEM)
     sprintf( pidstr, "%10d\n", (int)getpid() );
     /* fixme: add the hostname to the second line (FQDN or IP addr?) */
 
@@ -116,8 +118,17 @@ create_dotlock( const char *file_to_lock )
     else
 	nodename = utsbuf.nodename;
 
-    if( !(dirpart = strrchr( file_to_lock, '/' )) ) {
-	dirpart = ".";
+#ifdef __riscos__
+    {
+        char *iter = (char *) nodename;
+        for (; iter[0]; iter++)
+            if (iter[0] == '.')
+                iter[0] = '/';
+    }
+#endif /* __riscos__ */
+
+    if( !(dirpart = strrchr( file_to_lock, DIRSEP_C )) ) {
+	dirpart = EXTSEP_S;
 	dirpartlen = 1;
     }
     else {
@@ -125,15 +136,20 @@ create_dotlock( const char *file_to_lock )
 	dirpart = file_to_lock;
     }
 
-  #ifdef _REENTRANT
+#ifdef _REENTRANT
     /* fixme: aquire mutex on all_lockfiles */
-  #endif
+#endif
     h->next = all_lockfiles;
     all_lockfiles = h;
 
-    h->tname = m_alloc( dirpartlen + 6+30+ strlen(nodename) + 11 );
+    h->tname = xmalloc( dirpartlen + 6+30+ strlen(nodename) + 11 );
+#ifndef __riscos__
     sprintf( h->tname, "%.*s/.#lk%p.%s.%d",
-	     dirpartlen, dirpart, h, nodename, (int)getpid() );
+	     dirpartlen, dirpart, (void *)h, nodename, (int)getpid() );
+#else /* __riscos__ */
+    sprintf( h->tname, "%.*s.lk%p/%s/%d",
+	     dirpartlen, dirpart, (void *)h, nodename, (int)getpid() );
+#endif /* __riscos__ */
 
     do {
 	errno = 0;
@@ -144,43 +160,82 @@ create_dotlock( const char *file_to_lock )
 	all_lockfiles = h->next;
 	log_error( "failed to create temporary file `%s': %s\n",
 					    h->tname, strerror(errno));
-	m_free(h->tname);
-	m_free(h);
+	xfree(h->tname);
+	xfree(h);
 	return NULL;
     }
     if( write(fd, pidstr, 11 ) != 11 ) {
 	all_lockfiles = h->next;
-      #ifdef _REENTRANT
+#ifdef _REENTRANT
 	/* release mutex */
-      #endif
+#endif
 	log_fatal( "error writing to `%s': %s\n", h->tname, strerror(errno) );
 	close(fd);
 	unlink(h->tname);
-	m_free(h->tname);
-	m_free(h);
+	xfree(h->tname);
+	xfree(h);
 	return NULL;
     }
     if( close(fd) ) {
 	all_lockfiles = h->next;
-      #ifdef _REENTRANT
+#ifdef _REENTRANT
 	/* release mutex */
-      #endif
+#endif
 	log_error( "error closing `%s': %s\n", h->tname, strerror(errno));
 	unlink(h->tname);
-	m_free(h->tname);
-	m_free(h);
+	xfree(h->tname);
+	xfree(h);
 	return NULL;
     }
 
-  #ifdef _REENTRANT
+#ifdef _REENTRANT
     /* release mutex */
-  #endif
-#endif /* !HAVE_DOSISH_SYSTEM */
-    h->lockname = m_alloc( strlen(file_to_lock) + 6 );
-    strcpy(stpcpy(h->lockname, file_to_lock), ".lock");
+#endif
+#endif
+    h->lockname = xmalloc( strlen(file_to_lock) + 6 );
+    strcpy(stpcpy(h->lockname, file_to_lock), EXTSEP_S "lock");
     return h;
 }
 
+
+void
+destroy_dotlock ( DOTLOCK h )
+{
+#if !defined (HAVE_DOSISH_SYSTEM)
+    if ( h )
+      {
+        DOTLOCK hprev, htmp;
+
+        /* First remove the handle from our global list of all locks. */
+        for (hprev=NULL, htmp=all_lockfiles; htmp; hprev=htmp, htmp=htmp->next)
+          if (htmp == h)
+            {
+              if (hprev)
+                hprev->next = htmp->next;
+              else
+                all_lockfiles = htmp->next;
+              h->next = NULL;
+              break;
+            }
+
+        /* Second destroy the lock. */
+	if (!h->disable)
+          {
+	    if (h->locked && h->lockname)
+              unlink (h->lockname);
+            if (h->tname)
+              unlink (h->tname);
+	    xfree (h->tname);
+	    xfree (h->lockname);
+          }
+	xfree(h);
+
+      }
+#endif
+}
+
+#ifndef HAVE_DOSISH_SYSTEM
+
 static int
 maybe_deadlock( DOTLOCK h )
 {
@@ -193,133 +248,12 @@ maybe_deadlock( DOTLOCK h )
     return 0;
 }
 
-/****************
- * Do a lock on H. A TIMEOUT of 0 returns immediately,
- * -1 waits forever (hopefully not), other
- * values are timeouts in milliseconds.
- * Returns: 0 on success
- */
-int
-make_dotlock( DOTLOCK h, long timeout )
-{
-#ifdef HAVE_DOSISH_SYSTEM
-    return 0;
-#else
-    int  pid;
-    const char *maybe_dead="";
-    int backoff=0;
-
-    if( h->disable ) {
-	return 0;
-    }
-
-    if( h->locked ) {
-	log_debug("oops, `%s' is already locked\n", h->lockname );
-	return 0;
-    }
-
-    for(;;) {
-	if( !link(h->tname, h->lockname) ) {
-	    /* fixme: better use stat to check the link count */
-	    h->locked = 1;
-	    return 0; /* okay */
-	}
-	if( errno != EEXIST ) {
-	    log_error( "lock not made: link() failed: %s\n", strerror(errno) );
-	    return -1;
-	}
-	if( (pid = read_lockfile(h->lockname)) == -1 ) {
-	    if( errno != ENOENT ) {
-		log_info("cannot read lockfile\n");
-		return -1;
-	    }
-	    log_info( "lockfile disappeared\n");
-	    continue;
-	}
-	else if( pid == getpid() ) {
-	    log_info( "Oops: lock already hold by us\n");
-	    h->locked = 1;
-	    return 0; /* okay */
-	}
-	else if( kill(pid, 0) && errno == ESRCH ) {
-	    maybe_dead = " - probably dead";
-	 #if 0 /* we should not do this without checking the permissions */
-	       /* and the hostname */
-	    log_info( "removing stale lockfile (created by %d)", pid );
-	 #endif
-	}
-	if( timeout == -1 ) {
-	    struct timeval tv;
-	    log_info( "waiting for lock (hold by %d%s) %s...\n",
-		      pid, maybe_dead, maybe_deadlock(h)? "(deadlock?) ":"");
-
-
-	    /* can't use sleep, cause signals may be blocked */
-	    tv.tv_sec = 1 + backoff;
-	    tv.tv_usec = 0;
-	    select(0, NULL, NULL, NULL, &tv);
-	    if( backoff < 10 )
-		backoff++ ;
-	}
-	else
-	    return -1;
-    }
-    /*not reached */
-#endif /* !HAVE_DOSISH_SYSTEM */
-}
-
-
-/****************
- * release a lock
- * Returns: 0 := success
- */
-int
-release_dotlock( DOTLOCK h )
-{
-#ifdef HAVE_DOSISH_SYSTEM
-    return 0;
-#else
-    int pid;
-
-    if( h->disable ) {
-	return 0;
-    }
-
-    if( !h->locked ) {
-	log_debug("oops, `%s' is not locked\n", h->lockname );
-	return 0;
-    }
-
-    pid = read_lockfile( h->lockname );
-    if( pid == -1 ) {
-	log_error( "release_dotlock: lockfile error\n");
-	return -1;
-    }
-    if( pid != getpid() ) {
-	log_error( "release_dotlock: not our lock (pid=%d)\n", pid);
-	return -1;
-    }
-    if( unlink( h->lockname ) ) {
-	log_error( "release_dotlock: error removing lockfile `%s'",
-							h->lockname);
-	return -1;
-    }
-    /* fixme: check that the link count is now 1 */
-    h->locked = 0;
-    return 0;
-#endif /* !HAVE_DOSISH_SYSTEM */
-}
-
-
 /****************
  * Read the lock file and return the pid, returns -1 on error.
  */
 static int
 read_lockfile( const char *name )
 {
-  #ifdef HAVE_DOSISH_SYSTEM
-    return 0;
-  #else
     int fd, pid;
     char pidstr[16];
 
@@ -338,20 +272,174 @@ read_lockfile( const char *name )
     pidstr[10] = 0;  /* terminate pid string */
     close(fd);
     pid = atoi(pidstr);
+#ifndef __riscos__
     if( !pid || pid == -1 ) {
+#else /* __riscos__ */
+    if( (!pid && riscos_getpid()) || pid == -1 ) {
+#endif /* __riscos__ */
 	log_error("invalid pid %d in lockfile `%s'", pid, name );
 	errno = 0;
 	return -1;
     }
     return pid;
-  #endif
+}
+#endif /* !HAVE_DOSISH_SYSTEM */
+
+/****************
+ * Do a lock on H. A TIMEOUT of 0 returns immediately,
+ * -1 waits forever (hopefully not), other
+ * values are timeouts in milliseconds.
+ * Returns: 0 on success
+ */
+int
+make_dotlock( DOTLOCK h, long timeout )
+{
+#if defined (HAVE_DOSISH_SYSTEM)
+    return 0;
+#else
+    int  pid;
+    const char *maybe_dead="";
+    int backoff=0;
+
+    if( h->disable ) {
+	return 0;
+    }
+
+    if( h->locked ) {
+#ifndef __riscos__
+	log_debug("oops, `%s' is already locked\n", h->lockname );
+#endif /* !__riscos__ */
+	return 0;
+    }
+
+    for(;;) {
+#ifndef __riscos__
+	if( !link(h->tname, h->lockname) ) {
+	    /* fixme: better use stat to check the link count */
+	    h->locked = 1;
+	    return 0; /* okay */
+	}
+	if( errno != EEXIST ) {
+	    log_error( "lock not made: link() failed: %s\n", strerror(errno) );
+	    return -1;
+	}
+#else /* __riscos__ */
+        if( !riscos_renamefile(h->tname, h->lockname) ) {
+            h->locked = 1;
+            return 0; /* okay */
+        }
+        if( errno != EEXIST ) {
+	    log_error( "lock not made: rename() failed: %s\n", strerror(errno) );
+	    return -1;
+        }
+#endif /* __riscos__ */
+	if( (pid = read_lockfile(h->lockname)) == -1 ) {
+	    if( errno != ENOENT ) {
+		log_info("cannot read lockfile\n");
+		return -1;
+	    }
+	    log_info( "lockfile disappeared\n");
+	    continue;
+	}
+	else if( pid == getpid() ) {
+	    log_info( "Oops: lock already held by us\n");
+	    h->locked = 1;
+	    return 0; /* okay */
+	}
+	else if( kill(pid, 0) && errno == ESRCH ) {
+#ifndef __riscos__
+	    maybe_dead = " - probably dead";
+#if 0 /* we should not do this without checking the permissions */
+	       /* and the hostname */
+	    log_info( "removing stale lockfile (created by %d)", pid );
+#endif
+#else /* __riscos__ */
+            /* we are *pretty* sure that the other task is dead and therefore
+               we remove the other lock file */
+            maybe_dead = " - probably dead - removing lock";
+            unlink(h->lockname);
+#endif /* __riscos__ */
+	}
+	if( timeout == -1 ) {
+	    struct timeval tv;
+	    log_info( "waiting for lock (held by %d%s) %s...\n",
+		      pid, maybe_dead, maybe_deadlock(h)? "(deadlock?) ":"");
+
+
+	    /* can't use sleep, cause signals may be blocked */
+	    tv.tv_sec = 1 + backoff;
+	    tv.tv_usec = 0;
+	    select(0, NULL, NULL, NULL, &tv);
+	    if( backoff < 10 )
+		backoff++ ;
+	}
+	else
+	    return -1;
+    }
+    /*not reached */
+#endif
 }
 
 
+/****************
+ * release a lock
+ * Returns: 0 := success
+ */
+int
+release_dotlock( DOTLOCK h )
+{
+#if defined (HAVE_DOSISH_SYSTEM)
+    return 0;
+#else
+    int pid;
+
+    /* To avoid atexit race conditions we first check whether there
+       are any locks left.  It might happen that another atexit
+       handler tries to release the lock while the atexit handler of
+       this module already ran and thus H is undefined.  */
+    if(!all_lockfiles)
+        return 0;
+
+    if( h->disable ) 
+	return 0;
+
+    if( !h->locked ) {
+	log_debug("oops, `%s' is not locked\n", h->lockname );
+	return 0;
+    }
+
+    pid = read_lockfile( h->lockname );
+    if( pid == -1 ) {
+	log_error( "release_dotlock: lockfile error\n");
+	return -1;
+    }
+    if( pid != getpid() ) {
+	log_error( "release_dotlock: not our lock (pid=%d)\n", pid);
+	return -1;
+    }
+#ifndef __riscos__
+    if( unlink( h->lockname ) ) {
+	log_error( "release_dotlock: error removing lockfile `%s'",
+							h->lockname);
+	return -1;
+    }
+#else /* __riscos__ */
+    if( riscos_renamefile(h->lockname, h->tname) ) {
+	log_error( "release_dotlock: error renaming lockfile `%s' to `%s'",
+							h->lockname, h->tname);
+	return -1;
+    }
+#endif /* __riscos__ */
+    /* fixme: check that the link count is now 1 */
+    h->locked = 0;
+    return 0;
+#endif
+}
+
 void
 remove_lockfiles()
 {
-  #ifndef HAVE_DOSISH_SYSTEM
+#if !defined (HAVE_DOSISH_SYSTEM)
     DOTLOCK h, h2;
 
     h = all_lockfiles;
@@ -359,16 +447,8 @@ remove_lockfiles()
 
     while( h ) {
 	h2 = h->next;
-	if( !h->disable ) {
-	    if( h->locked )
-		unlink( h->lockname );
-	    unlink(h->tname);
-	    m_free(h->tname);
-	    m_free(h->lockname);
-	}
-	m_free(h);
+        destroy_dotlock (h);
 	h = h2;
     }
-  #endif
+#endif
 }
-
