@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -165,6 +166,7 @@ init_ttyfp(void)
 #ifdef HAVE_LIBREADLINE
     rl_catch_signals = 0;
     rl_instream = rl_outstream = ttyfp;
+    rl_inhibit_completion = 1;
 #endif
 #endif
 #ifdef HAVE_TCGETATTR
@@ -173,6 +175,33 @@ init_ttyfp(void)
     initialized = 1;
 }
 
+#ifdef HAVE_LIBREADLINE
+void
+tty_enable_completion(rl_completion_func_t *completer)
+{
+  if( no_terminal )
+    return;
+
+  if( !initialized )
+    init_ttyfp();
+
+  rl_attempted_completion_function=completer;
+  rl_inhibit_completion=0;
+}
+
+void
+tty_disable_completion(void)
+{
+  if( no_terminal )
+    return;
+
+  if( !initialized )
+    init_ttyfp();
+
+  rl_inhibit_completion=1;
+}
+#endif /*HAVE_LIBREADLINE*/
+
 int
 tty_batchmode( int onoff )
 {
@@ -217,7 +246,7 @@ tty_printf( const char *fmt, ... )
 	if( n != nwritten )
 	    log_fatal ("WriteConsole failed: %d != %d\n", n, (int)nwritten );
 	last_prompt_len += n;
-        m_free (buf);
+        xfree (buf);
     }
 #else
     last_prompt_len += vfprintf(ttyfp,fmt,arg_ptr) ;
@@ -336,7 +365,7 @@ tty_print_utf8_string2 (const byte *p, size_t n, size_t max_n )
 	}
 	/*(utf8 conversion already does the control character quoting)*/
 	tty_printf("%s", buf );
-	m_free( buf );
+	xfree( buf );
     }
     else {
 	if( max_n && (n > max_n) ) {
@@ -376,14 +405,15 @@ do_get( const char *prompt, int hidden )
 	init_ttyfp();
 
     last_prompt_len = 0;
-    tty_printf( "%s", prompt );
-    buf = m_alloc(n=50);
+    buf = xmalloc(n=50);
     i = 0;
 
 #ifdef _WIN32 /* windoze version */
     if( hidden )
 	SetConsoleMode(con.in, HID_INPMODE );
 
+    tty_printf( "%s", prompt );
+
     for(;;) {
 	DWORD nread;
 
@@ -406,7 +436,7 @@ do_get( const char *prompt, int hidden )
 	    continue;
 	if( !(i < n-1) ) {
 	    n += 50;
-	    buf = m_realloc( buf, n );
+	    buf = xrealloc( buf, n );
 	}
 	buf[i++] = c;
     }
@@ -415,6 +445,7 @@ do_get( const char *prompt, int hidden )
 	SetConsoleMode(con.in, DEF_INPMODE );
 
 #elif defined(__riscos__)
+    tty_printf( "%s", prompt );
     do {
         c = riscos_getchar();
         if (c == 0xa || c == 0xd) { /* Return || Enter */
@@ -444,7 +475,7 @@ do_get( const char *prompt, int hidden )
         }
         if(!(i < n-1)) {
             n += 50;
-            buf = m_realloc(buf, n);
+            buf = xrealloc(buf, n);
         }
         buf[i++] = c;
         if (!hidden) {
@@ -469,6 +500,8 @@ do_get( const char *prompt, int hidden )
 #endif
     }
 
+    tty_printf( "%s", prompt );
+
     /* fixme: How can we avoid that the \n is echoed w/o disabling
      * canonical mode - w/o this kill_prompt can't work */
     while( read(fileno(ttyfp), cbuf, 1) == 1 && *cbuf != '\n' ) {
@@ -486,7 +519,7 @@ do_get( const char *prompt, int hidden )
 	    continue;
 	if( !(i < n-1) ) {
 	    n += 50;
-	    buf = m_realloc( buf, n );
+	    buf = xrealloc( buf, n );
 	}
 	buf[i++] = c;
     }
@@ -527,7 +560,7 @@ tty_get( const char *prompt )
       /* We need to copy it to memory controlled by our malloc
          implementations; further we need to convert an EOF to our
          convention. */
-      buf = m_alloc(line? strlen(line)+1:2);
+      buf = xmalloc(line? strlen(line)+1:2);
       if (line)
         {
           strcpy (buf, line);
@@ -591,6 +624,6 @@ tty_get_answer_is_yes( const char *prompt )
     char *p = tty_get( prompt );
     tty_kill_prompt();
     yes = answer_is_yes(p);
-    m_free(p);
+    xfree(p);
     return yes;
 }
